@font-face{font-family:rawline;font-style:normal;font-weight:400;src:local('Rawline'),url(./fonts/rawline-400.woff)}
@font-face{font-family:rawline-italic;font-style:italic;font-weight:400;src:local('Rawline'),url(./fonts/rawline-500i.woff)}
@font-face{font-family:rawline-bold;font-weight:bold;font-weight:400;src:local('Rawline'),url(./fonts/rawline-700.woff)}
body{
    margin: 0px;
    background-color: "white";
    font-family: rawline;
    color: #383838;
}
h1, h2, h3, h4, h5, dt, strong, .bold{
    font-family: rawline-bold;
}
a {
    color: inherit;
    text-decoration: none;
}

.form-control{
    font-family: rawline;
    font-size: 15px;
    -webkit-appearance: none;
    color: inherit;
    background-color: inherit;
}

.form-control:focus {
    outline: none !important;
    border: solid 1px #c2850c !important;
}
input[type="checkbox"], input[type="radio"]{
    appearance: none;
    background-color: #fff;
    margin: 0;
    font: inherit;
    width: 1.5em;
    height: 1.5em;
    border: solid 1px #CCC;
    transform: translateY(-0.075em);
    display: inline-grid;
    place-content: center;
    margin-right: 10px;
  }
  input[type="checkbox"]{
      border-radius: 0.15em;
  }
  input[type="radio"]{
      border-radius: 50%;
  }
  input[type="checkbox"]::before, input[type="radio"]::before {
    content: "";
    width: 0.65em;
    height: 0.65em;
    transform: scale(0);
    transition: 120ms transform ease-in-out;
    box-shadow: inset 1em 1em var(--primary-color);
  }
  input[type="checkbox"]:checked::before {
    transform: scale(1);
  }
  input[type="radio"]:checked::before {
    transform: scale(1);
    border-radius: 50%;
  }

  form.login {
    width: 350px;
  }
  form.login h1{
    text-align: center;
  }


  /* .object-viewer .object-fieldset {
    border: solid 1px #2c3e50;
    border-radius: 5px;
  }

  .object-viewer .object-fieldset .fieldset-title{
    background-color: #2c3e50;
    color: white;
    padding: 5px;
  } */
